package com.qucheng.game.data.oldsystem.ods;

import com.qucheng.game.data.oldsystem.Env;
import com.qucheng.game.data.oldsystem.pojo.FlinkAppConfigParam;
import com.qucheng.game.data.oldsystem.pojo.MysqlConfigParam;
import com.qucheng.game.data.oldsystem.pojo.TransportMap;
import com.qucheng.game.data.oldsystem.serialization.MapDebeziumDeserializationSchema;
import com.qucheng.game.data.oldsystem.sink.CdcMysqlTablesSink;
import com.ververica.cdc.connectors.mysql.source.MySqlSource;
import com.ververica.cdc.connectors.mysql.table.StartupOptions;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.api.common.restartstrategy.RestartStrategies;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.runtime.state.hashmap.HashMapStateBackend;
import org.apache.flink.streaming.api.CheckpointingMode;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

public class OldGameSystemCDCBgNew {
    private static final FlinkAppConfigParam appConfigParam = Env.isTest ? FlinkAppConfigParam.builder()
            .savePath("")
            .interval(300L)
            .timeout(300L)
            .minBetween(1L)
            .build() : FlinkAppConfigParam.builder()
            .savePath("hdfs://nameservice1:8020/user/limeng/cluster_yarn/checkpoints/ODS/" + OldGameSystemCDCBgNew.class.getSimpleName())
            .interval(300L)
            .timeout(300L)
            .minBetween(1L)
            .build();

    public static void main(String[] args) throws Exception {
        System.setProperty("HADOOP_USER_NAME", "flink");

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        // 加载配置文件到 flink的全局配置中
        Properties props = new Properties();
        props.load(OldGameSystemCDCBgNew.class.getResourceAsStream(Env.isTest ? "/application.test.properties" : "/application.properties"));
        Configuration configuration = new Configuration();
        props.stringPropertyNames().forEach(key -> {
            String value = props.getProperty(key);
            configuration.setString(key.trim(), StringUtils.isBlank(value) ? "" : value.trim());
        });
        env.getConfig().setGlobalJobParameters(configuration);
        // 设置默认并行度
        env.setParallelism(1);

        // 任务失败后的重启策略
        // env.setRestartStrategy(RestartStrategies.noRestart());// 失败不重启
        // env.setRestartStrategy(RestartStrategies.fixedDelayRestart(3, Time.of(10, TimeUnit.SECONDS)));// 3:最大重试次数、10:重启间隔时间
        env.setRestartStrategy(RestartStrategies.failureRateRestart(3, Time.of(5, TimeUnit.MINUTES), Time.of(10, TimeUnit.SECONDS)));// 5分钟内最多重启 3次，重启间隔时间：10s

        // checkpoint配置
        env.enableCheckpointing(appConfigParam.getInterval() * 1000, CheckpointingMode.EXACTLY_ONCE);
        // checkpoint执行超时时间，超时则 checkpoint失败
        env.getCheckpointConfig().setCheckpointTimeout(appConfigParam.getTimeout() * 1000);
        // checkpoint执行最小间隔时间
        env.getCheckpointConfig().setMinPauseBetweenCheckpoints(appConfigParam.getMinBetween() * 1000);
        // 允许并行执行 checkpoint个数，当指定了 minPauseBetweenCheckpoints时，其值无效（就是 1）
        env.getCheckpointConfig().setMaxConcurrentCheckpoints(1);
        // 开启 checkpoints的外部持久化，但是在job失败的时候不会自动清理，需要自己手工清理state。
        // ExternalizedCheckpointCleanup用于指定当job canceled的时候外部的 checkpoint该如何清理
        // DELETE_ON_CANCELLATION: 在job canceled的时候会自动删除外部 state，但是如果是FAILED的状态则会保留
        // RETAIN_ON_CANCELLATION：在job canceled的时候保留外部 state
        env.getCheckpointConfig().enableExternalizedCheckpoints(CheckpointConfig.ExternalizedCheckpointCleanup.RETAIN_ON_CANCELLATION);
        // 设置可容忍的检查点失败数，默认值为0表示不允许容忍任何检查点失败
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(2);
        // 设置状态后端
        env.setStateBackend(new HashMapStateBackend());
        // 设置检查点目录
        if (StringUtils.isNotBlank(appConfigParam.getSavePath())) {
            env.getCheckpointConfig().setCheckpointStorage(appConfigParam.getSavePath());
        }

        MySqlSource<TransportMap> mysqlCDCSource = MySqlSource.<TransportMap>builder()
                .hostname(props.getProperty("cdc.mysql.backup.host"))
                .port(StringUtils.isBlank(props.getProperty("cdc.mysql.backup.port")) ? 3306 : Integer.parseInt(props.getProperty("cdc.mysql.backup.port")))
                .username(props.getProperty("cdc.mysql.backup.username"))
                .password(props.getProperty("cdc.mysql.backup.password"))
                .databaseList("bg_new_sdk_db_mp")
                .tableList((StringUtils.join(new String[]{
                        "bg_new_sdk_db_mp.h_game",
                        "bg_new_sdk_db_mp.h_mg_role",
                        "bg_new_sdk_db_mp.h_pay",
                        "bg_new_sdk_db_mp.h_pay_ext",
                        "bg_new_sdk_db_mp.qc_h_mem_game",
                        "bg_new_sdk_db_mp.qc_h_member",
                        "bg_new_sdk_db_mp.qc_h_user"
                }, ",")))
                .deserializer(new MapDebeziumDeserializationSchema())
                .startupOptions(StartupOptions.latest())
                .build();

        MysqlConfigParam sinkMysqlConfig = MysqlConfigParam.builder()
                .url(props.getProperty("mysql.oldGameSystemBgNewOds.url"))
                .username(props.getProperty("mysql.oldGameSystemBgNewOds.username"))
                .password(props.getProperty("mysql.oldGameSystemBgNewOds.password"))
                .build();

        env.fromSource(mysqlCDCSource, WatermarkStrategy.noWatermarks(), "mysqlCDCSource")
                .map(new FieldFilterMap())
                .addSink(new CdcMysqlTablesSink(sinkMysqlConfig));

        env.execute(OldGameSystemCDCBgNew.class.getSimpleName());
    }

    /**
     * 字段过滤和映射
     */
    public static class FieldFilterMap implements MapFunction<TransportMap, TransportMap> {
        @Override
        public TransportMap map(TransportMap transportMap) throws Exception {
            if (transportMap.getAfter() == null) {
                return transportMap;
            }
            Map<String, Object> after = transportMap.getAfter();
            if (transportMap.getTableName().equalsIgnoreCase("h_game")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList("id",
                        "name",
                        "en_name",
                        "en_abbr",
                        "app_key",
                        "tags",
                        "category",
                        "classify",
                        "icon",
                        "cp_payback_url",
                        "cp_id",
                        "parent_id",
                        "package_name",
                        "pay_switch",
                        "order_switch",
                        "pay_show",
                        "float_is_show",
                        "status",
                        "is_delete",
                        "delete_time",
                        "is_online",
                        "is_sdk",
                        "list_order",
                        "rise_order",
                        "hot_order",
                        "like_order",
                        "publicity",
                        "language",
                        "description",
                        "image",
                        "run_time",
                        "create_time",
                        "update_time",
                        "fine_order",
                        "promote_switch",
                        "apple_id",
                        "add_cp_time",
                        "is_bt",
                        "is_auth",
                        "single_tag")) {
                    pojo.put(field, after.get(field));
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("qc_h_mem_game")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList(
                        "id",
                        "mem_id",
                        "guided_agent_id",
                        "app_id",
                        "nickname",
                        "create_time",
                        "update_time",
                        "is_default",
                        "like",
                        "star_cnt",
                        "status",
                        "sum_money")) {
                    pojo.put(field, after.get(field));
                    transportMap.setTableName("h_mem_game");
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("qc_h_member")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList(
                        "id",
                        "nickname",
                        "from_device",
                        "device_id",
                        "app_id",
                        "agent_id",
                        "status",
                        "create_time",
                        "update_time",
                        "username")) {
                    pojo.put(field, after.get(field));
                    transportMap.setTableName("h_member");
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("h_mg_role")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList(
                        "id",
                        "mg_mem_id",
                        "app_id",
                        "server_id",
                        "server_name",
                        "role_id",
                        "role_name",
                        "role_level",
                        "role_vip",
                        "money",
                        "combat_num",
                        "ext",
                        "os",
                        "create_time",
                        "update_time"
                )) {
                    pojo.put(field, after.get(field));
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("h_pay")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList(
                        "id",
                        "order_id",
                        "cp_order_id",
                        "mem_id",
                        "mg_mem_id",
                        "agent_id",
                        "app_id",
                        "currency",
                        "amount",
                        "real_amount",
                        "product_id",
                        "product_cnt",
                        "product_name",
                        "coupon_amount",
                        "ptb_amount",
                        "gm_amount",
                        "integral",
                        "integral_money",
                        "rebate_amount",
                        "rate",
                        "status",
                        "cp_status",
                        "payway",
                        "mobile_prefix",
                        "is_handle",
                        "pay_time",
                        "create_time",
                        "update_time",
                        "is_distribute",
                        "notify_cnt",
                        "last_notify_time",
                        "ext",
                        "mem_note",
                        "admin_note",
                        "remark",
                        "is_switch",
                        "vb_id")) {
                    pojo.put(field, after.get(field));
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("h_pay_ext")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList("pay_id",
                        "product_id",
                        "product_name",
                        "product_desc",
                        "product_cnt",
                        "device_id",
                        "mac",
                        "ip",
                        "brand",
                        "model",
                        "os",
                        "os_version",
                        "screen",
                        "net",
                        "imsi",
                        "longitude",
                        "latitude",
                        "userua",
                        "server_id",
                        "server_name",
                        "role_id",
                        "role_name",
                        "role_level",
                        "money",
                        "role_vip",
                        "online_time",
                        "scene",
                        "axis",
                        "last_operation",
                        "party_name",
                        "role_balance",
                        "experience",
                        "attach",
                        "type",
                        "rolelevel_ctime",
                        "rolelevel_mtime")) {
                    pojo.put(field, after.get(field));
                }
                after = pojo;
            } else if (transportMap.getTableName().equalsIgnoreCase("qc_h_user")) {
                Map<String, Object> pojo = new HashMap<>(after.size());
                for (String field : Arrays.asList("id",
                        "user_login",
                        "user_nicename",
                        "create_time",
                        "role_id",
                        "parent_id",
                        "mem_id",
                        "account_id",
                        "parent_account_id",
                        "cp_id")) {
                    pojo.put(field, after.get(field));
                    transportMap.setTableName("h_user");
                }
                after = pojo;
            }
            transportMap.setAfter(after);
            return transportMap;
        }
    }
}
